<?php

namespace GtmEcommerceWooPro\Lib\EventStrategy\Browser;

use GtmEcommerceWoo\Lib\EventStrategy\PurchaseStrategy as FreePurchaseStrategy;
use WC_Order;

class PurchaseStrategy extends FreePurchaseStrategy {

	const LOCAL_STORAGE_KEY_PATTERN_PURCHASE_EVENT_TRACKED = 'gtm_ecommerce_woo_purchase_event_tracked_%d';

	/**
	 * Tracked order.
	 *
	 * @var WC_Order
	 */
	private $trackedOrder;

	public function defineActions() {
		return [
			'woocommerce_thankyou' => [$this, 'thankyou'],
			'woocommerce_pay_order_before_submit' => [$this, 'formPay'],
			'woocommerce_before_checkout_form' => [$this, 'checkoutForm'],
			'woocommerce_checkout_order_processed' => [[$this, 'ajaxCheckout'], 10, 3],
			'wp_footer' => [$this, 'wpFooter'],
		];
	}

	public function thankyou( $orderId ) {
		$order = wc_get_order( $orderId );

		if (false === $order instanceof WC_Order) {
			return;
		}

		if (false === $this->shouldTrack($order)) {
			return;
		}

		$this->trackOrder($order);
	}

	public function formPay() {
		global $wp;

		if (false === isset($wp->query_vars['order-pay'])) {
			return;
		}

		$orderId = $wp->query_vars['order-pay'];

		$order = wc_get_order( $orderId );

		if (false === $order instanceof WC_Order) {
			return;
		}

		if (false === $this->shouldTrack($order)) {
			return;
		}

		$this->trackOrder($order);
	}

	public function ajaxCheckout( $orderId, $postedData, WC_Order $order) {
		if (false === $this->shouldTrack($order)) {
			return;
		}

		$event = $this->wcTransformer->getPurchaseFromOrder($order);

		$serializedEvent = base64_encode(json_encode($event));

		add_filter( 'woocommerce_payment_successful_result', static function( $result) use ( $serializedEvent ) {
			$result['gtm_ecommerce_woo_purchase_event'] = $serializedEvent;

			return $result;
		});
	}

	public function checkoutForm() {
		$this->wcOutput->script(
			<<<EOD
jQuery(document).on('ajaxSuccess', function(event, xhr, settings) {
	try {
		if (false === settings.url.includes('wc-ajax=checkout')) {
			return;
		}

		const responseObject = {
			result: null,
			gtm_ecommerce_woo_purchase_event: null,
			...JSON.parse(xhr.responseText)
		};

		if ('success' !== responseObject.result || null === responseObject.gtm_ecommerce_woo_purchase_event) {
			return;
		}

		const purchaseEvent = JSON.parse(atob(responseObject.gtm_ecommerce_woo_purchase_event));

		dataLayer.push(purchaseEvent);

		const orderId = purchaseEvent.ecommerce.transaction_id;

		localStorage.setItem('{$this->getLocalStorageKeyPatternPurchaseEventTracked()}'.replace('%d', orderId), '1');
	} catch (e) {
		return;
	}
});
EOD
		);
	}

	public function wpFooter() {
		if (null === $this->trackedOrder) {
			return;
		}

		$this->trackedOrder->update_meta_data(self::ORDER_META_KEY_PURCHASE_EVENT_TRACKED, '1');
		$this->trackedOrder->save();
	}

	private function shouldTrack( WC_Order $order) {
		return $this->isTrackingForced() || '1' !== $order->get_meta(self::ORDER_META_KEY_PURCHASE_EVENT_TRACKED);
	}

	private function trackOrder( WC_Order $order ) {
		$event = $this->wcTransformer->getPurchaseFromOrder($order);
		$stringifiedEvent = json_encode($event);

		$trackingForced = $this->isTrackingForced() ? 'true' : 'false';
		$localStorageKey = sprintf(self::LOCAL_STORAGE_KEY_PATTERN_PURCHASE_EVENT_TRACKED, $order->get_id());

		$this->wcOutput->script(
			<<<EOD
		if ({$trackingForced} || null === localStorage.getItem('{$localStorageKey}')) {
			dataLayer.push({$stringifiedEvent});
		}
EOD
		);

		$this->trackedOrder = $order;
	}

	private function getLocalStorageKeyPatternPurchaseEventTracked() {
		return self::LOCAL_STORAGE_KEY_PATTERN_PURCHASE_EVENT_TRACKED;
	}

	private function isTrackingForced() {
		return ( isset($_GET['gtm-ecommerce-woo-force-purchase'] ) && '1' === $_GET['gtm-ecommerce-woo-force-purchase'] );
	}
}
